==> v0.13.11.tar.gz: git-next/crates/cli/src/server/tests.rs <==
//
use assert2::let_assert;
use git_next_core::{
    self as core,
    git::{self, repository::Direction, validation::remotes::validate_default_remotes},
    ApiToken, ForgeType, GitDir, Hostname, RepoBranches, RepoConfig, RepoConfigSource, RepoPath,
    StoragePathType, User,
};
use secrecy::Secret;

type Result<T> = std::result::Result<T, Box<dyn std::error::Error>>;

#[test]
fn test_repo_config_load() -> Result<()> {
    let toml = r#"[branches]
            main = "main"
            next = "next"
            dev = "dev"

            [options]
        "#;
    let config = RepoConfig::parse(toml)?;

    assert_eq!(
        config,
        RepoConfig::new(
            RepoBranches::new("main".to_string(), "next".to_string(), "dev".to_string(),),
            RepoConfigSource::Repo
        )
    );

    Ok(())
}

#[test]
fn gitdir_should_display_as_pathbuf() {
    //given
    let gitdir = GitDir::new("foo/dir".into(), StoragePathType::External);
    //when
    let result = format!("{gitdir}");
    //then
    assert_eq!(result, "foo/dir");
}

#[test]
// NOTE: this test assumes it is being run in a cloned worktree from the project's home repo:
// git.kemitix.net:kemitix/git-next
// If the default push remote is something else, then this test will fail
fn repo_details_find_default_push_remote_finds_correct_remote() -> Result<()> {
    let cli_crate_dir = std::env::current_dir().map_err(git::validation::remotes::Error::Io)?;
    let_assert!(Some(Some(root)) = cli_crate_dir.parent().map(|p| p.parent()));
    let mut repo_details = git::repo_details(
        1,
        git::Generation::default(),
        core::common::forge_details(1, ForgeType::MockForge),
        None,
        GitDir::new(root.to_path_buf(), StoragePathType::External), // Server GitDir - should be ignored
    );
    repo_details.forge = repo_details
        .forge
        .with_user(User::new("git".to_string()))
        .with_token(ApiToken::new(Secret::new(String::new())))
        .with_hostname(Hostname::new("git.kemitix.net"));
    repo_details.repo_path = RepoPath::new("kemitix/git-next".to_string());
    let Ok(open_repository) = git::repository::factory::real().open(&repo_details) else {
        // .git directory may not be present on dev environment
        return Ok(());
    };
    let_assert!(
        Some(found_git_remote) = open_repository.find_default_remote(Direction::Push),
        "Default Push Remote not found"
    );
    let_assert!(Some(config_git_remote) = repo_details.remote_url());

    assert!(
        found_git_remote.matches(&config_git_remote),
        "Default Push Remote must match config"
    );

    Ok(())
}

#[test_log::test]
fn gitdir_validate_should_pass_a_valid_git_repo() -> Result<()> {
    let cli_crate_dir = std::env::current_dir().map_err(git::validation::remotes::Error::Io)?;
    let_assert!(Some(Some(root)) = cli_crate_dir.parent().map(|p| p.parent()));
    let mut repo_details = git::repo_details(
        1,
        git::Generation::default(),
        core::common::forge_details(1, ForgeType::MockForge),
        None,
        GitDir::new(root.to_path_buf(), StoragePathType::External), // Server GitDir - should be ignored
    )
    .with_repo_path(RepoPath::new("kemitix/git-next".to_string()));
    repo_details.forge = repo_details
        .forge
        .with_user(User::new("git".to_string()))
        .with_token(ApiToken::new(Secret::new(String::new())))
        .with_hostname(Hostname::new("git.kemitix.net"));
    tracing::debug!("opening...");
    let Ok(repository) = git::repository::factory::real().open(&repo_details) else {
        // .git directory may not be present on dev environment
        return Ok(());
    };
    tracing::debug!("open okay");
    tracing::info!(?repository, "FOO");
    tracing::info!(?repo_details, "BAR");
    validate_default_remotes(&*repository, &repo_details)?;

    Ok(())
}

#[test]
fn gitdir_validate_should_fail_a_git_repo_with_wrong_remote() {
    let_assert!(
        Ok(cli_crate_dir) = std::env::current_dir().map_err(git::validation::remotes::Error::Io)
    );
    eprintln!("cli_crate_dir: {cli_crate_dir:?}");
    let_assert!(Some(Some(root)) = cli_crate_dir.parent().map(|p| p.parent()));
    eprintln!("root: {root:?}");
    let mut repo_details = git::repo_details(
        1,
        git::Generation::default(),
        core::common::forge_details(1, ForgeType::MockForge),
        None,
        GitDir::new(root.to_path_buf(), StoragePathType::External), // Server GitDir - should be ignored
    )
    .with_repo_path(RepoPath::new("kemitix/git-next".to_string()));
    repo_details.forge = repo_details
        .forge
        .with_user(User::new("git".to_string()))
        .with_token(ApiToken::new(Secret::new(String::new())))
        .with_hostname(Hostname::new("git.kemitix.net"));
    let Ok(repository) = git::repository::factory::real().open(&repo_details) else {
        // .git directory may not be present on dev environment
        return;
    };
    let mut repo_details = repo_details.clone();
    repo_details.forge = repo_details
        .forge
        .with_hostname(Hostname::new("code.kemitix.net"));
    let_assert!(Err(_) = validate_default_remotes(&*repository, &repo_details));
}

#[test]
fn git_remote_to_string_is_as_expected() {
    let git_remote = git::GitRemote::new(Hostname::new("foo"), RepoPath::new("bar".to_string()));
    let as_string = git_remote.to_string();

    assert_eq!(as_string, "foo:bar");
}

==> v0.14.1.tar.gz: git-next/crates/cli/src/server/tests.rs <==
//
use assert2::let_assert;
use git_next_core::{
    self as core,
    git::{self, repository::Direction, validation::remotes::validate_default_remotes},
    ApiToken, ForgeType, GitDir, Hostname, RepoBranches, RepoConfig, RepoConfigSource, RepoPath,
    StoragePathType, User,
};
use secrecy::SecretString;

type Result<T> = std::result::Result<T, Box<dyn std::error::Error>>;

#[test]
fn test_repo_config_load() -> Result<()> {
    let toml = r#"[branches]
            main = "main"
            next = "next"
            dev = "dev"

            [options]
        "#;
    let config = RepoConfig::parse(toml)?;

    assert_eq!(
        config,
        RepoConfig::new(
            RepoBranches::new("main".to_string(), "next".to_string(), "dev".to_string(),),
            RepoConfigSource::Repo
        )
    );

    Ok(())
}

#[test]
fn gitdir_should_display_as_pathbuf() {
    //given
    let gitdir = GitDir::new("foo/dir".into(), StoragePathType::External);
    //when
    let result = format!("{gitdir}");
    //then
    assert_eq!(result, "foo/dir");
}

#[test]
// NOTE: this test assumes it is being run in a cloned worktree from the project's home repo:
// git.kemitix.net:kemitix/git-next
// If the default push remote is something else, then this test will fail
fn repo_details_find_default_push_remote_finds_correct_remote() -> Result<()> {
    let cli_crate_dir = std::env::current_dir().map_err(git::validation::remotes::Error::Io)?;
    let_assert!(Some(Some(root)) = cli_crate_dir.parent().map(|p| p.parent()));
    let mut repo_details = git::repo_details(
        1,
        git::Generation::default(),
        core::common::forge_details(1, ForgeType::MockForge),
        None,
        GitDir::new(root.to_path_buf(), StoragePathType::External), // Server GitDir - should be ignored
    );
    repo_details.forge = repo_details
        .forge
        .with_user(User::new("git".to_string()))
        .with_token(ApiToken::new(SecretString::from(String::new())))
        .with_hostname(Hostname::new("git.kemitix.net"));
    repo_details.repo_path = RepoPath::new("kemitix/git-next".to_string());
    let Ok(open_repository) = git::repository::factory::real().open(&repo_details) else {
        // .git directory may not be present on dev environment
        return Ok(());
    };
    let_assert!(
        Some(found_git_remote) = open_repository.find_default_remote(Direction::Push),
        "Default Push Remote not found"
    );
    let_assert!(Some(config_git_remote) = repo_details.remote_url());

    assert!(
        found_git_remote.matches(&config_git_remote),
        "Default Push Remote must match config"
    );

    Ok(())
}

#[test_log::test]
fn gitdir_validate_should_pass_a_valid_git_repo() -> Result<()> {
    let cli_crate_dir = std::env::current_dir().map_err(git::validation::remotes::Error::Io)?;
    let_assert!(Some(Some(root)) = cli_crate_dir.parent().map(|p| p.parent()));
    let mut repo_details = git::repo_details(
        1,
        git::Generation::default(),
        core::common::forge_details(1, ForgeType::MockForge),
        None,
        GitDir::new(root.to_path_buf(), StoragePathType::External), // Server GitDir - should be ignored
    )
    .with_repo_path(RepoPath::new("kemitix/git-next".to_string()));
    repo_details.forge = repo_details
        .forge
        .with_user(User::new("git".to_string()))
        .with_token(ApiToken::new(SecretString::from(String::new())))
        .with_hostname(Hostname::new("git.kemitix.net"));
    tracing::debug!("opening...");
    let Ok(repository) = git::repository::factory::real().open(&repo_details) else {
        // .git directory may not be present on dev environment
        return Ok(());
    };
    tracing::debug!("open okay");
    tracing::info!(?repository, "FOO");
    tracing::info!(?repo_details, "BAR");
    validate_default_remotes(&*repository, &repo_details)?;

    Ok(())
}

#[test]
fn gitdir_validate_should_fail_a_git_repo_with_wrong_remote() {
    let_assert!(
        Ok(cli_crate_dir) = std::env::current_dir().map_err(git::validation::remotes::Error::Io)
    );
    eprintln!("cli_crate_dir: {cli_crate_dir:?}");
    let_assert!(Some(Some(root)) = cli_crate_dir.parent().map(|p| p.parent()));
    eprintln!("root: {root:?}");
    let mut repo_details = git::repo_details(
        1,
        git::Generation::default(),
        core::common::forge_details(1, ForgeType::MockForge),
        None,
        GitDir::new(root.to_path_buf(), StoragePathType::External), // Server GitDir - should be ignored
    )
    .with_repo_path(RepoPath::new("kemitix/git-next".to_string()));
    repo_details.forge = repo_details
        .forge
        .with_user(User::new("git".to_string()))
        .with_token(ApiToken::new(SecretString::from(String::new())))
        .with_hostname(Hostname::new("git.kemitix.net"));
    let Ok(repository) = git::repository::factory::real().open(&repo_details) else {
        // .git directory may not be present on dev environment
        return;
    };
    let mut repo_details = repo_details.clone();
    repo_details.forge = repo_details
        .forge
        .with_hostname(Hostname::new("code.kemitix.net"));
    let_assert!(Err(_) = validate_default_remotes(&*repository, &repo_details));
}

#[test]
fn git_remote_to_string_is_as_expected() {
    let git_remote = git::GitRemote::new(Hostname::new("foo"), RepoPath::new("bar".to_string()));
    let as_string = git_remote.to_string();

    assert_eq!(as_string, "foo:bar");
}
